@@ -60,7 +60,7 @@ lod_candles = 3000
 lod_labels = 700
 cache_candle_factor = 3 # factor extra candles rendered to buffer
 y_label_width = 65
-long_time = 2*365*24*60*60*1000
+long_time = 2*365*24*60*60*1e9
 display_timezone = None  # default to local
 winx,winy,winw,winh = 400,300,800,400
 
@@ -68,7 +68,6 @@ app = None
 windows = [] # no gc
 timers = [] # no gc
 sounds = {} # no gc
-plotdf2df = {} # for pandas df.plot
 epoch_period = 1e30
 last_ax = None # always assume we want to plot in the last axis, unless explicitly specified
 overlay_axs = [] # for keeping track of candlesticks in overlays
@@ -120,6 +119,8 @@ class YAxisItem(pg.AxisItem):
         super().__init__(*args, **kwargs)
         self.vb = vb
         self.hide_strings = False
+        self.style['autoExpandTextSpace'] = False
+        self.style['autoReduceTextSpace'] = False
 
     def tickStrings(self, values, scale, spacing):
         if self.hide_strings:
@@ -194,7 +195,7 @@ class PandasDataSource:
             return 1
         if not self._period:
             timecol = self.df.columns[0]
-            self._period = self.df[timecol].diff().median() / 1000
+            self._period = self.df[timecol].diff().median() / 1e9
         return self._period
 
     @property
@@ -355,22 +356,6 @@ class PandasDataSource:
 
 
 
-class PlotDf(object):
-    '''This class is for allowing you to do df.plot(...), as you normally would in Pandas.'''
-    def __init__(self, df):
-        global plotdf2df
-        plotdf2df[self] = df
-    def __getattribute__(self, name):
-        if name == 'plot':
-            return partial(dfplot, plotdf2df[self])
-        return getattr(plotdf2df[self], name)
-    def __getitem__(self, i):
-        return plotdf2df[self].__getitem__(i)
-    def __setitem__(self, i, v):
-        return plotdf2df[self].__setitem__(i, v)
-
-
-
 class FinWindow(pg.GraphicsLayoutWidget):
     def __init__(self, title, **kwargs):
         global winx, winy
@@ -393,6 +378,7 @@ class FinWindow(pg.GraphicsLayoutWidget):
     def close(self):
         self.closing = True
         _savewindata(self)
+        _clear_timers()
         return super().close()
 
     def eventFilter(self, obj, ev):
@@ -648,6 +634,7 @@ class FinViewBox(pg.ViewBox):
         ev.accept()
 
     def mouseDragEvent(self, ev, axis=None):
+        axis = 0 # don't constrain drag direction
         if self.master_viewbox:
             return self.master_viewbox.mouseDragEvent(ev, axis=axis)
         if not self.datasrc:
@@ -656,10 +643,13 @@ class FinViewBox(pg.ViewBox):
             self.mouseLeftDrag(ev, axis)
         elif ev.button() == QtCore.Qt.MiddleButton:
             self.mouseMiddleDrag(ev, axis)
+        elif ev.button() == QtCore.Qt.RightButton:
+            self.mouseRightDrag(ev, axis)
         else:
             super().mouseDragEvent(ev, axis)
 
     def mouseLeftDrag(self, ev, axis):
+        '''Ctrl+LButton draw lines.'''
         if ev.modifiers() != QtCore.Qt.ControlModifier:
             super().mouseDragEvent(ev, axis)
             if ev.isFinish() or self.drawing:
@@ -687,6 +677,7 @@ class FinViewBox(pg.ViewBox):
         ev.accept()
 
     def mouseMiddleDrag(self, ev, axis):
+        '''Ctrl+MButton draw ellipses.'''
         if ev.modifiers() != QtCore.Qt.ControlModifier:
             return super().mouseDragEvent(ev, axis)
         p1 = self.mapToView(ev.pos())
@@ -695,7 +686,7 @@ class FinViewBox(pg.ViewBox):
             c = b-a
             return pg.Point(abs(c.x()) or 1, abs(c.y()) or 1e-3)
         if not self.drawing:
-            # add new line
+            # add new ellipse
             p0 = self.mapToView(ev.lastPos())
             p0 = _clamp_point(self.parent(), p0)
             s = nonzerosize(p0, p1)
@@ -713,6 +704,19 @@ class FinViewBox(pg.ViewBox):
             self.drawing = False
         ev.accept()
 
+    def mouseRightDrag(self, ev, axis):
+        '''RButton is box zoom. At least for now.'''
+        ev.accept()
+        if not ev.isFinish():
+            self.updateScaleBox(ev.buttonDownPos(), ev.pos())
+        else:
+            self.rbScaleBox.hide()
+            ax = QtCore.QRectF(pg.Point(ev.buttonDownPos(ev.button())), pg.Point(ev.pos()))
+            ax = self.childGroup.mapRectFromParent(ax)
+            self.showAxRect(ax)
+            self.axHistoryPointer += 1
+            self.axHistory = self.axHistory[:self.axHistoryPointer] + [ax]
+
     def mouseClickEvent(self, ev):
         if self.master_viewbox:
             return self.master_viewbox.mouseClickEvent(ev)
@@ -1273,7 +1277,7 @@ def renko(x, y=None, bins=None, step=None, ax=None, colorfunc=price_colorfilter)
     item.colors['bull_body'] = item.colors['bull_frame']
     item.update_data = partial(_update_data, None, step_adjust_renko_datasrc, item)
     global epoch_period
-    epoch_period = (origdf.iloc[1,0] - origdf.iloc[0,0]) // 1000
+    epoch_period = (origdf.iloc[1,0] - origdf.iloc[0,0]) // int(1e9)
     return item
 
 
@@ -1453,11 +1457,11 @@ def fill_between(plot0, plot1, color=None):
     return item
 
 
-def dfplot(df, x=None, y=None, color=None, width=1, ax=None, style=None, legend=None, zoomscale=True):
-    legend = legend if legend else y
-    x = x if x else df.columns[0]
-    y = y if y else df.columns[1]
-    return plot(df[x], df[y], color=color, width=width, ax=ax, style=style, legend=legend, zoomscale=zoomscale)
+def set_x_pos(xmin, xmax, ax=None):
+    ax = _create_plot(ax=ax, maximize=False)
+    xidx0,xidx1 = _pdtime2index(ax, pd.Series([xmin, xmax]))
+    ax.vb.update_y_zoom(xidx0, xidx1)
+    _repaint_candles()
 
 
 def set_y_range(ymin, ymax, ax=None):
@@ -1585,7 +1589,7 @@ def show(qt_exec=True):
         app.exec_()
         windows.clear()
         overlay_axs.clear()
-        timers.clear()
+        _clear_timers()
         sounds.clear()
         master_data.clear()
         last_ax = None
@@ -1674,6 +1678,12 @@ def _create_plot(ax=None, **kwargs):
     return create_plot(**kwargs)
 
 
+def _clear_timers():
+    for timer in timers:
+        timer.timeout.disconnect()
+    timers.clear()
+
+
 def _add_timestamp_plot(master, prev_ax, viewbox, index, yscale):
     native_win = isinstance(master, pg.GraphicsLayoutWidget)
     if native_win and prev_ax is not None:
@@ -1686,7 +1696,7 @@ def _add_timestamp_plot(master, prev_ax, viewbox, index, yscale):
         axw = pg.PlotWidget(viewBox=viewbox, axisItems=axes, name='plot-%i'%index)
         ax = axw.plotItem
         ax.ax_widget = axw
-    ax.axes['left']['item'].textWidth = y_label_width # this is to put all graphs on equal footing when texts vary from 0.4 to 2000000
+    ax.axes['left']['item'].setWidth(y_label_width) # this is to put all graphs on equal footing when texts vary from 0.4 to 2000000
     ax.axes['left']['item'].setStyle(tickLength=-5) # some bug, totally unexplicable (why setting the default value again would fix repaint width as axis scale down)
     ax.axes['left']['item'].setZValue(30) # put axis in front instead of behind data
     ax.axes['bottom']['item'].setZValue(30)
@@ -1732,6 +1742,7 @@ def _overlay(ax, scale=0.25, y_axis=False):
     axo.decouple = partial(_ax_decouple, axo)
     axo.disable_x_index = partial(_ax_disable_x_index, axo)
     axo.hideAxis('left')
+    axo.hideAxis('right')
     axo.hideAxis('bottom')
     axo.hideButtons()
     viewbox.addItem(axo)
@@ -1754,9 +1765,9 @@ def _ax_set_visible(ax, crosshair=None, xaxis=None, yaxis=None, yaxis_labels=Non
     if crosshair == False:
         ax.crosshair.hide()
     if xaxis is not None:
-        (ax.showAxis if xaxis else ax.hideAxis)('bottom')
+        ax.showAxis('bottom', xaxis)
     if yaxis is not None:
-        (ax.showAxis if yaxis else ax.hideAxis)('left')
+        ax.showAxis('left', yaxis)
     if yaxis_labels is not None:
         ax.axes['left']['item'].hide_strings = not yaxis_labels
 
@@ -1954,6 +1965,9 @@ def _update_data(preadjustfunc, adjustfunc, item, ds):
     ds = _create_datasrc(item.ax, ds)
     if adjustfunc:
         adjustfunc(ds)
+    cs = list(item.datasrc.df.columns[:1]) + list(item.datasrc.df.columns[item.datasrc.col_data_offset:])
+    if len(cs) >= len(ds.df.columns):
+        ds.df.columns = cs[:len(ds.df.columns)]
     item.datasrc.update(ds)
     _start_visual_update(item)
     for i in item.ax.items:
@@ -2039,9 +2053,10 @@ def _repaint_candles():
     '''Candles are only partially drawn, and therefore needs manual dirty reminder whenever it goes off-screen.'''
     axs = [ax for win in windows for ax in win.axs] + overlay_axs
     for ax in axs:
-        for item in ax.items:
+        for item in list(ax.items):
             if isinstance(item, FinPlotItem):
-                item.repaint()
+                _start_visual_update(item)
+                _end_visual_update(item)
 
 
 def _paint_scatter(item, p, *args):
@@ -2166,10 +2181,13 @@ def _get_color(ax, style, wanted_color):
 def _pdtime2epoch(t):
     if isinstance(t, pd.Series):
         if isinstance(t.iloc[0], pd.Timestamp):
-            return t.astype('int64') // int(1e6)
-        if np.nanmax(t.values) > 1e13: # handle ns epochs
-            return (t/1e3).astype('int64')
-        if np.nanmax(t.values) < 1e10: # handle s epochs
+            return t.astype('int64')
+        h = np.nanmax(t.values)
+        if h < 1e10: # handle s epochs
+            return (t*1e9).astype('int64')
+        if h < 1e13: # handle ns epochs
+            return (t*1e6).astype('int64')
+        if h < 1e16: # handle us epochs
             return (t*1e3).astype('int64')
         return t.astype('int64')
     return t
@@ -2177,16 +2195,18 @@ def _pdtime2epoch(t):
 
 def _pdtime2index(ax, ts, any_end=False, require_time=False):
     if isinstance(ts.iloc[0], pd.Timestamp):
-        ts = ts.astype('int64') // int(1e6)
+        ts = ts.astype('int64')
     else:
         h = np.nanmax(ts.values)
         if h < 1e7:
             if require_time:
                 assert False, 'not a time series'
             return ts
-        if h > 1e13: # handle ns epochs
-            ts = ts.astype('float64') / 1e3
-        elif h < 1e10: # handle s epochs
+        if h < 1e10: # handle s epochs
+            ts = ts.astype('float64') * 1e9
+        elif h < 1e13: # handle ms epochs
+            ts = ts.astype('float64') * 1e6
+        elif h < 1e16: # handle us epochs
             ts = ts.astype('float64') * 1e3
     
     datasrc = _get_datasrc(ax)
@@ -2229,12 +2249,16 @@ def _get_datasrc(ax, require=True):
         assert ax.vb.datasrc, 'not possible to plot this primitive without a prior time-range to compare to'
 
 
+def _millisecond_wrap(s):
+    return (s+'.000000') if '.' not in s else s
+
+
 def _x2local_t(datasrc, x):
-    return _x2t(datasrc, x, lambda t: datetime.fromtimestamp(t/1000, tz=display_timezone).isoformat(sep=' ').partition('+')[0])
+    return _x2t(datasrc, x, lambda t: _millisecond_wrap(datetime.fromtimestamp(t/1e9, tz=display_timezone).isoformat(sep=' ').partition('+')[0]))
 
 
 def _x2utc(datasrc, x):
-    return _x2t(datasrc, x, lambda t: datetime.utcfromtimestamp(t/1000).isoformat(sep=' '))
+    return _x2t(datasrc, x, lambda t: _millisecond_wrap(datetime.utcfromtimestamp(t/1e9).isoformat(sep=' ')))
 
 
 def _x2t(datasrc, x, ts2str):
@@ -2247,6 +2271,7 @@ def _x2t(datasrc, x, ts2str):
             if not datasrc.timebased():
                 return '%g' % t, False
             s = ts2str(t)
+            
             if epoch_period >= 23*60*60: # daylight savings, leap seconds, etc
                 i = s.index(' ')
             elif epoch_period >= 59: # consider leap seconds
@@ -2326,7 +2351,7 @@ def _draw_line_segment_text(polyline, segment, pos0, pos1):
             t0,_,_,_,cnt0 = datasrc.hilo(x0, x0)
             t1,_,_,_,cnt1 = datasrc.hilo(x1, x1)
             if cnt0 and cnt1:
-                fsecs = abs(t1 - t0) / 1000
+                fsecs = abs(t1 - t0) / 1e9
         except:
             pass
     diff = pos1 - pos0
@@ -2400,7 +2425,7 @@ def _makepen(color, style=None, width=1):
 
 try:
     qtver = '%d.%d' % (QtCore.QT_VERSION//256//256, QtCore.QT_VERSION//256%256)
-    if qtver not in ('5.9', '5.13'):
+    if qtver not in ('5.9', '5.13') and [int(i) for i in pg.__version__.split('.')] <= [0,11,0]:
         print('WARNING: your version of Qt may not plot curves containing NaNs and is not recommended.')
         print('See https://github.com/pyqtgraph/pyqtgraph/issues/1057')
 except:
@@ -2409,6 +2434,8 @@ except:
 
 # default to black-on-white
 pg.widgets.GraphicsView.GraphicsView.wheelEvent = partialmethod(_wheel_event_wrapper, pg.widgets.GraphicsView.GraphicsView.wheelEvent)
+# use finplot instead of matplotlib
+pd.set_option('plotting.backend', 'finplot.pdplot')
 # pick up win resolution
 try:
     import ctypes
