@@ -15,7 +15,7 @@ interval = '1d'
 url = 'https://query1.finance.yahoo.com/v7/finance/download/%s?period1=%s&period2=%s&interval=%s&events=history' % (symbol, start_t, end_t, interval)
 r = requests.get(url)
 df = pd.read_csv(StringIO(r.text))
-df['Date'] = pd.to_datetime(df['Date']).astype('int64') // 1_000_000 # use finplot's internal representation, which is ms
+df['Date'] = pd.to_datetime(df['Date']).astype('int64') # use finplot's internal representation, which is ns
 
 ax,ax2 = fplt.create_plot('S&P 500 MACD', rows=2)
 
